"""
Author: B. Chen

Spatial attention network implementation.
"""

from .wrapper import SpatialAttentionWrapper
from .config import SpatialAttentionConfig

__all__ = ['SpatialAttentionWrapper', 'SpatialAttentionConfig']